#include "GameOverState.hpp"
#include "Utility.hpp"

GameOverState::GameOverState(StateStack& stack, Context context)
	: State(stack, context)
	, bg(sf::Vector2f(1280, 600))
	, last("", context.fonts->Get(Font::Emulator), 30)
	, best("", context.fonts->Get(Font::Emulator))
	, startAgain("Press SPACE to play again!", context.fonts->Get(Font::Emulator), 30) {

	CenterOrigin(startAgain);
	startAgain.setPosition(640, 450);
	startAgain.setColor(sf::Color());

	std::stringstream ss; ss << "Your score: " << lastScore;
	last.setString(ss.str());
	last.setColor(sf::Color());

	ss = std::stringstream();
	if(lastScore > bestScore) {
		bestScore = lastScore;

		ss << "New best score: " << bestScore << "!";
		best.setString(ss.str());
		best.setColor(sf::Color(255, 134, 32));
		best.setCharacterSize(50);
	} else {
		ss << "Best score: " << bestScore;
		best.setString(ss.str());
		best.setColor(sf::Color());
	}

	CenterOrigin(last);
	CenterOrigin(best);

	last.setPosition(640, 150);
	best.setPosition(640, 250);

	bg.setFillColor(sf::Color::White);
}

bool GameOverState::HandleEvent(const sf::Event& e) {
	switch(e.type) {
	case sf::Event::KeyPressed:
		if(e.key.code == sf::Keyboard::Space) {
			RequestStackPop();
			RequestStackPush(States::GAMESTATE);
		}
		break;
	}

	return true;
}

bool GameOverState::Update(sf::Time& dt) {

	return true;
}

void GameOverState::Draw(sf::RenderTarget& target, sf::RenderStates states) {
	target.draw(bg);
	target.draw(last);
	target.draw(best);
	target.draw(startAgain);
}